use serde_derive::{Deserialize, Serialize};
use serde_xml_rs::{from_reader, from_str, to_string};

use clap::Parser;

// use std::fmt;
use std::fs::File;

// If I don't use group here, it isn't available to sudoku module
// If I use it here, I get warning about unused module.

// mod group;
// use crate::group::*;
use sudoku::*;

// mod sudoku;
// use crate::sudoku::*;
// use crate::sudoku::group::*;

// use crate::group::*;
// use crate::sudoku::*;

#[derive(Debug, Serialize, Deserialize, PartialEq)]
struct Graph {
    order: i32,
    #[serde(rename = "type")]
    game_type: String,
    #[serde(rename = "specific-type")]
    specific_type: String,
}

#[derive(Debug, Serialize, Deserialize, PartialEq)]
struct Puzzle {
    graph: Graph,
    values: String,
    solution: String,
}

#[derive(Debug, Serialize, Deserialize, PartialEq)]
struct Game {
    #[serde(rename = "had-help")]
    help: i16,
    #[serde(rename = "msecs-elapsed")]
    elapsed: u32,
    puzzle: Puzzle,
}
#[derive(Debug, Serialize, Deserialize, PartialEq)]
struct Ksudoku {
    game: Game,
}


// Command line arguments - via clap

#[derive(Parser)]
struct Cli {
    /// The path to the file to read
    filename: std::path::PathBuf,
}

fn main() {
    let args = Cli::parse();

    let filename = args.filename; // "../puzzle1";
    let fh = File::open(filename).unwrap();
    let puzzle: Ksudoku = from_reader(fh).unwrap();
    println!("Puzzle: {:?}", puzzle);

    let mut s = Sudoku::new();
    s.load_xsudoku(&puzzle.game.puzzle.values);
    s.display();
    s.display_possible();

    while s.solve() {
        println!("Try it again...");
        s.display();
        s.display_possible();
    }

    /*
    s.display();
    s.display_possible();
    */

    /*
    let mut g = Group::new();
    g.for_row(1, 1);
    print!("Row: ");
    println!("{:?}", g);
    println!("{:#?}", g);
    // g.display();
    print!("Col: ");
    g.for_column(1, 1);
    println!("{:?}", g);
    // g.display();
    println!("Blk testing output: ");

    for i in 0..SIZE {
        g.for_block(i, i);
        println!("({},{} {:#?})", i, i, g);
        // g.display();
    }
    */
}
